import time
import winsound
from typing import Iterable


class Beep:

    def __init__(self):
        pass

    @staticmethod
    def beep(frequency: int, duration: int):
        """

        :param frequency: frequency of the beep
        :type frequency: int
        :param duration: duration in millisecond
        :type duration: int
        :return:
        """
        winsound.Beep(frequency=frequency, duration=duration)

    @staticmethod
    def morse_executor(code: Iterable[int],
                       frequency: int = 600,
                       short_duration: int = 400,
                       long_duration: int = 800,
                       delay: float = 0.1):
        """
        morse code generator
        1: long beep
        0: short beep

        :param delay:
        :param long_duration:
        :param short_duration:
        :param frequency:
        :param code:
        :return: None
        """

        if not isinstance(code, Iterable):
            raise TypeError("Use int type iterable!")
        for letter in code:
            if letter == 1:
                print("1 chosen")
                Beep.beep(frequency=frequency, duration=long_duration)
                # time.sleep(0.01)
            elif letter == 0:
                print("0 chosen")
                Beep.beep(frequency=frequency, duration=short_duration)
                time.sleep(delay)
            else:
                print("Error at: {}".format(letter))
                raise TypeError("Use numbers 0 or 1")


    @staticmethod
    def split(word: str) -> list:
        """

        :param word: string to split. e.g.: '110011100'
        :type word: str
        :return: list of integers
        :rtype: list
        """
        string_list = [char for char in word]
        for i in range(len(string_list)):
            if type(string_list[i]) != int:
                raise TypeError
            string_list[i] = int(string_list[i])
        return string_list

    @staticmethod
    def morse(code: Iterable[int]):
        print("morse str: {}, {}".format(type(code), str(code)))
        Beep.morse_executor(code=code)


if __name__ == "__main__":
    print("Starting")
    # Beep.morse("1001100")
    Beep.morse_executor((1, 0, 1, 1, 0, 0, 1))
    Beep.morse_executor([0, 0, 0, 1, 1, 1, 0, 0, 0])
    Beep.morse_executor("asdasd")
    print("Finished")



import unittest

import calculator


class BasicTestSuite(unittest.TestCase):
    def test_something(self):
        self.assertEqual(calculator.Calculator.get_average([1, 2]), 1.5)


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from pathlib import Path
from unittest import TestCase

from .csvmanager import CSVManager


class TestCSVManager(TestCase):
    i = CSVManager(str(Path(os.getcwd()).parent))

    def test_check_csv_integrity_false(self):
        self.assertFalse(self.i.check_csv_for_empty_cells(
            [['a', 'b', 'c'],
             ['1', '', '3'],
             ['', '2', '3'],
             ['1', '', ''],
             ['1', '2', '3']]))

    def test_check_csv_integrity_true(self):
        self.assertTrue(self.i.check_csv_for_empty_cells([['a', 'b', 'c'],
                                                          ['1', '2', '3']]))

    def test_check_csv_for_diff_types_true(self):
        self.assertTrue(self.i.check_csv_for_diff_types([['a', 'b', 'c'],
                                                         ['x', 0, 'i'],
                                                         ['y', 1, 'j']]))

    def test_check_csv_for_diff_types_false(self):
        self.assertFalse(self.i.check_csv_for_diff_types([['a', 'b'],
                                                          ['x', 0],
                                                          ['y', 'i']]))


if __name__ == '__main__':
    unittest.main()  # pragma: no cover



"""
Managing csv files with data type(float, str) validator
"""

import csv
import logging
import sys
from os import listdir
from os.path import isfile, join

import pandas as pd


class CSVManager:
    """
    CSVManager is able to handle easily csv files with validation.
    """

    csv_path: str
    csv_files: list
    logger = logging.getLogger('Importer')
    handler = logging.StreamHandler(sys.stdout)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    def __init__(self, current_path: str):
        self.csv_path = current_path
        self.logger.setLevel(logging.DEBUG)
        self.handler.setLevel(logging.INFO)
        self.handler.setFormatter(self.formatter)
        self.logger.addHandler(self.handler)

    def get_all_data_as_list(self):
        """
        Getting all the csv data as a list.
        :return: all the csv content in a list.
        """
        return self.read_all_csv()

    def get_csv_files(self):  # pragma: no cover
        """
        Get the list of csv file names

        :return: list of csv file names
        """

        module_path = self.csv_path
        csv_files = [f for f in listdir(module_path) if isfile(join(module_path, f))
                     and ".csv" in f]
        self.csv_files = list(csv_files)
        return csv_files

    def check_csv_for_diff_types(self, csv_reader: list):
        """
        Checking for different types in rows

        NOTE: only checking str and float types (no datetime and such)

        NOTE2: sale/incoming can be used as a boolean type

        :param csv_reader: csv file content
        :type csv_reader: list
        :return: Bool
        """

        self.logger.debug(csv_reader)
        # read content by rows without header
        dataframe = pd.DataFrame(csv_reader)
        columns = []
        for column_name in dataframe.columns.array:
            columns.append(dataframe[column_name].to_numpy())
        for column in columns:
            data_type = ''
            for cell in column:
                try:
                    float(cell)
                    data_type = 'float'
                except ValueError:
                    if data_type == 'float':
                        logging.error('cannot convert to a float: ' + str(cell))
                        return False
        return True

    def check_csv_for_empty_cells(self, csv_reader: list):
        """
        Checking for empty cells
        :param csv_reader: csv file content
        :type csv_reader: list
        :return: Bool: integrity
        """

        self.logger.debug(csv_reader)
        integrity = True
        for row in csv_reader:
            for cell in row:
                if cell in (None, ""):
                    logging.error("Empty cell found at: " + str(row))
                    integrity = False
        return integrity

    def read_all_csv(self):
        """
        reading all csv files
        :return: list: all the csv contents in a list
        """

        content = []
        try:
            self.get_csv_files()
        except AttributeError:
            self.get_csv_files()
        self.logger.debug(self.csv_files)
        for file in self.csv_files:
            print(file)
            content.append(self.read_csv_file(file=file))
        return content

    def read_csv_file(self, file: str):
        """
        read the content of a single csv file
        :param file: csv file name
        :type file: str
        :return: list: csv content
        """

        content = []
        with open(self.csv_path + "/" + file, 'r') as csv_file:
            csv_reader = csv.reader(csv_file)
            self.check_csv_for_empty_cells(csv_reader=list(csv_reader))
            self.check_csv_for_diff_types(csv_reader=list(csv_reader))

        with open(self.csv_path + "/" + file, 'r') as csv_file:
            for row in csv.reader(csv_file):
                content.append(row)
        return content



"""
General used methods for e.g. prevent screensaver

"""

from ctypes import windll


class SystemTools:

    @staticmethod
    def disable_lockscreen():
        """
        prevent screensaver/lock screen

        source:
        https://docs.microsoft.com/en-us/windows/win32/api/winbase/nf-winbase-setthreadexecutionstate

        """

        windll.kernel32.SetThreadExecutionState(0x80000002)

    @staticmethod
    def enable_lockscreen():
        """
        restore screensaver/lock screen

        source:
        https://docs.microsoft.com/en-us/windows/win32/api/winbase/nf-winbase-setthreadexecutionstate

        """

        windll.kernel32.SetThreadExecutionState(0x80000000)


if __name__ == "__main__":
    SystemTools.disable_lockscreen()
    input("Press enter to exit...")
    SystemTools.enable_lockscreen()



import multiprocessing.dummy
import subprocess


class Pinger:

    @staticmethod
    def ping(ip, liveoutput: bool = False):
        """

        :param liveoutput:
        :param ip:
        :return:
        """
        # get response time in ms or None if host not reached in 1 sec
        success = Pinger.singleping(hostname=ip)  # Run the ping command here
        if liveoutput:
            print("Live output enabled:")
            if success:
                print("{} responded".format(ip))
            else:
                print("{} did not respond".format(ip))
        return {ip: success}
        # return success

    @staticmethod
    def ping_range(network: str = "192.168.0.", start: int = 1, end: int = 254, liveoutput: bool = False):
        """

        :param liveoutput: is console output live, or just get the result at the end
        :type liveoutput: bool
        :param network: subnet (currently only /24 is implemented)
        :type network: str
        :param start: first host number
        :type start: int
        :param end: last host number
        :type end: int
        """
        # TODO: https://stackoverflow.com/questions/5442910/python-multiprocessing-pool-map-for-multiple-arguments
        from itertools import product
        num_threads = 2 * multiprocessing.cpu_count()

        with multiprocessing.Pool(processes=num_threads) as pool:
            results = pool.starmap(Pinger.ping, product([network + str(x) for x in range(start, end)]), liveoutput)

        return results

    @staticmethod
    def singleping(hostname: str):
        """
        Ping only 1 times

        :param hostname: hostname to ping
        :type hostname: str
        :return: result of ping: str
        """
        output = subprocess.getoutput("timeout 1 ping -c 1 " + hostname).split(" ")
        for i in output:
            if "time=" in i:
                response = i.split("=")
                return response[1]


if __name__ == "__main__":
    print(Pinger.ping_range(network="192.168.0.", start=1, end=81, liveoutput=True))



"""
Simple port scan tool
"""

import socket
import sys


class PortScan:

    @staticmethod
    def port_scan(host: str, port: int):
        ip = socket.gethostbyname(host)

        if port == "full":
            print("Executing full port scan. It can take 2-5 minutes...")
            try:
                # will scan ports between 1 to 65,535
                for port_number in range(1, 65535):
                    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    socket.setdefaulttimeout(1)

                    # returns an error indicator
                    scan_result = s.connect_ex((ip, port_number))
                    s.close()
                    if scan_result == 0:
                        print("Port {} is open".format(port_number))
                        return True
                    else:
                        print("Port {} is closed".format(port_number))
                        return False

            except KeyboardInterrupt:
                print("\n Exiting Program!!!")
                sys.exit(1)
            except socket.gaierror:
                print("\n Hostname Could Not Be Resolved!!!")
                sys.exit(1)
            except socket.error:
                print("\n Server not responding!!!")
                sys.exit(1)
        else:
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                socket.setdefaulttimeout(1)

                # returns an error indicator
                scan_result = s.connect_ex((ip, port))
                s.close()
                if scan_result == 0:
                    print("Port {} is open".format(port))
                    return True
                else:
                    print("Port {} is closed".format(port))
                    return False
            except KeyboardInterrupt:
                print("\n Exiting Program!!!")
                sys.exit(1)
            except socket.gaierror:
                print("\n Hostname Could Not Be Resolved!!!")
                sys.exit(1)
            except socket.error:
                print("\n Server not responding!!!")
                sys.exit(1)


if __name__ == '__main__':
    print(PortScan.port_scan("localhost", 1433))
    # print("PORT SCANNER")
    #
    # # Defining a target
    # HOST: str = ""
    # PORT: int = -1
    #
    # if len(sys.argv) != 3:
    #     print("Invalid amount of Argument")
    #     HOST = input("Give me the host name or IP: ")
    #     PORT = int(input("...and now the port(type 'full' if you want a full scan): "))
    # else:
    #     HOST = sys.argv[1]
    #     PORT = int(sys.argv[2])
    #
    # # translate hostname to IPv4
    # target = socket.gethostbyname(HOST)
    #
    # # Add Banner
    # print("-" * 50)
    # print("Scanning Target Host: " + HOST + " : " + target)
    # print("Scanning Target Port: " + str(PORT))
    # print("Scanning started at: " + str(datetime.now()))
    # print("-" * 50)
    #
    # if PORT == "full":
    #     print("Executing full port scan. It can take 2-5 minutes...")
    #     try:
    #         # will scan ports between 1 to 65,535
    #         for port in range(1, 65535):
    #             s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    #             socket.setdefaulttimeout(1)
    #
    #             # returns an error indicator
    #             result = s.connect_ex((target, port))
    #             if result == 0:
    #                 print("Port {} is open".format(port))
    #             s.close()
    #
    #     except KeyboardInterrupt:
    #         print("\n Exiting Program !!!!")
    #         sys.exit()
    #     except socket.gaierror:
    #         print("\n Hostname Could Not Be Resolved !!!!")
    #         sys.exit()
    #     except socket.error:
    #         print("\n Server not responding !!!!")
    #         sys.exit()
    # else:
    #     try:
    #         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    #         socket.setdefaulttimeout(1)
    #
    #         # returns an error indicator
    #         result = s.connect_ex((target, PORT))
    #         if result == 0:
    #             print("Port {} is open".format(PORT))
    #         else:
    #             print("Port {} is closed".format(PORT))
    #         s.close()
    #     except KeyboardInterrupt:
    #         print("\n Exiting Program !!!!")
    #         sys.exit()
    #     except socket.gaierror:
    #         print("\n Hostname Could Not Be Resolved !!!!")
    #         sys.exit()
    #     except socket.error:
    #         print("\n Server not responding !!!!")
    #         sys.exit()



"""General calculus operations"""

import numpy as np


class Calculator:
    """General calculus operations"""

    @staticmethod
    def get_average(numbers: list):
        """
        returns the average of the numbers in the list

        :param numbers: list of numbers
        :type numbers: list
        :return: float
        """

        return np.average(numbers)



import csv


class Reader:
    file: str = None
    delimiter: str = ','

    def __init__(self, file: str, delimiter: str = ','):
        self.file = file
        self.delimiter = delimiter

    def read(self):
        csv_header: list = []

        with open(file=self.file, mode="r") as csv_file:
            csv_reader = csv.reader(csv_file, delimiter=self.delimiter)
            line_count = 0
            for row in csv_reader:
                if line_count == 0:
                    print(f'Column names are {", ".join(row)}')
                    csv_header.append(row)
                else:
                    print(f'\t{row[0]} works in the {row[1]} department, and was born in {row[2]}.')
                line_count += 1
            print(f'Processed {line_count} lines.')


if __name__ == "__main__":
    import os


    def get_data(file_name: str):
        directory_name = os.path.dirname(__file__)
        directory_name = os.path.dirname(directory_name)
        return os.path.join(directory_name, "data/" + file_name)


    csv_example = Reader(file=get_data(file_name="example.csv"))
    csv_example.read()



"""
Massive selenium requests.
"""

from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as ec
from selenium.webdriver.support.wait import WebDriverWait


class Validreqs:
    """
    Validated selenium requests.
    """

    def __init__(self):
        pass

    @staticmethod
    def select_element(web_driver, element_name: str, element_type: str):
        """
        1. Check that it exists,
        2. select a web element,
        3. validate the selection,
        4. check for errors.

        :param web_driver: selenium webdriver instance
        :param element_name: name of the element. (id, class, name, xpath, css)
        :type element_name: str
        :param element_type: type of the element (id, class, name, xpath, css)
        :type element_type: str

        :return: bool: success of selection
        """
        WebDriverWait(web_driver, 20).until(ec.presence_of_element_located((By.CSS_SELECTOR, ".reply-button"))).click()

        pass


if __name__ == "__main__":
    driver = webdriver.Firefox(
        executable_path="C:\\Users\\nagyd\\Downloads\\geckodriver-v0.28.0-win64\\geckodriver.exe")
    driver.get("http://www.python.org")
    assert "Python" in driver.title
    elem = driver.find_element_by_name("q")
    elem.clear()
    elem.send_keys("pycon")
    elem.send_keys(Keys.RETURN)
    assert "No results found." not in driver.page_source

    Validreqs.select_element(web_driver=driver, element_name="about", element_type="id")

    driver.close()
